/* =========================================================[ Preferences.hh ]=
 * Preferences.hh - set player preferences
 * Copyright (C) 2003 Hubert Chan <hubert@uhoreg.ca>
 *
 * ----------------------------------------------------------------( License )-
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * ------------------------------------------------------------( Description )-
 *
 * This file maintains a set of user preferences, and allows the user to change
 * them via an on-screen menu.  The preferences have a simple model: each
 * preference has a name, and a discrete set of possible values.  The values
 * are ordered (possibly in a circular order), and have string representations.
 *
 * ------------------------------------------------------------------------- */

#ifndef __PREFERENCES_HH_
#define __PREFERENCES_HH_

#include <string>
#include <vector>

#include "SessionData.hh"

class Preferences
{
public:
  class Pref
  {
    std::string name;
  public:
    Pref(const std::string &_name) : name(_name) { }
    virtual ~Pref() { };

    const std::string &getName() const { return name; }
    virtual const std::string &getValue() const = 0;
    virtual void nextValue() = 0;
    virtual void prevValue() = 0;
  };

private:
  std::vector<Pref*> prefs;

public:
  Preferences(const SessionData &);
  ~Preferences();

  int getSize() const { return prefs.size(); }
  Pref &getItem(int i) const { return *(prefs[i]); }
};

#endif //__PREFERENCES_HH_
