/* =============================================================[ ap-lirc.cc ]=
 * ap-lirc.cc - LIRC interface plugin main file
 * Control AlsaPlayer through LIRC.
 * Copyright (C) 2002 Hubert Chan <hubert@uhoreg.ca>
 *
 * ----------------------------------------------------------------( License )-
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * ------------------------------------------------------------( Description )-
 *
 * This program allows one to control AlsaPlayer using a remote control through
 * LIRC (Linux Infrared Remote Control).
 *
 * Lots of code ripped from irexec.c and the AlsaPlayer text and gtk interface
 * plugins.  Commands based on lirc-xmms-plugin.
 *
 * ------------------------------------------------------------------------- */

#include <iostream>

#include "config.h"
#include "debug.hh"

#include <alsaplayer/AlsaPlayer.h>
#include <alsaplayer/SampleBuffer.h>
#include <alsaplayer/CorePlayer.h>
#include <alsaplayer/Playlist.h>
#include <alsaplayer/alsaplayer_error.h>
#include <alsaplayer/interface_plugin.h>
#include <alsaplayer/prefs.h>

#include <lirc/lirc_client.h>

#include "SessionData.hh"
#include "CommandSet.hh"

namespace {

  //------------------------------------------------------( Global variables )-
  /* note: global variables are evil.  I don't see any better way to do things,
   * though.  */

  struct lirc_config *config;

  //------------------------------------------------------( Plugin functions )-

  int interface_lirc_init()
  {
    DEBUG3 << "Entering lirc_init\n";

    if(lirc_init("alsaplayer",1)==-1)
    {
      std::cerr << "lirc interface error: could not initialize lirc\n";
      return 1; //?
    }

    if(lirc_readconfig(NULL,&config,NULL)!=0)
    {
      std::cerr << "lirc interface error: could not read configuration\n";
      lirc_deinit();
      return 1; //?
    }

    return 1;
  }

  int interface_lirc_running()
  {
    return 1;
  }

  int interface_lirc_stop()
  {
    return 1;
  }

  void interface_lirc_close()
  {
    DEBUG3 << "Entering lirc_close\n";

    lirc_freeconfig(config);
    lirc_deinit();
    return;
  }

  int interface_lirc_start(Playlist *playlist, int argc, char **argv)
  {
    DEBUG3 << "Entering lirc_start\n";

    SessionData sessiondata(playlist);

    char *code;
    char *c;
    char *args;
    int ret;
    CommandSet callbackset(sessiondata);

    while(lirc_nextcode(&code)==0)
    {
      DEBUG3 << "lirc_start input loop\n";
      if(code==NULL) continue;
      while((ret=lirc_code2char(config,code,&c))==0 &&
	    c!=NULL)
      {
	DEBUG3 << "lirc_start command loop\n";
	int i;
	for (i = 0; c[i] != '\0'; i++)
	  c[i] = tolower(c[i]);
	DEBUG2 << "command:" << c << '\n';

	c = strdup(c);
	/* separate command from arguments -- split at the first space */
	if ((args = strchr(c, ' ')) != NULL)
	{
	  *args = '\0';
	  args++;
	  while (*args == ' ')
	    args++;
	}

	if (strcmp(c, "quit") == 0)
	{
	  free(code);
	  return 0;
	}
	callbackset.call(c,args ? args : "");

	free(c);
      }

      free(code);
      if(ret==-1) break;
    }

    DEBUG3 << "Exiting lirc_start\n";
    return 0;
  }

  interface_plugin default_plugin =
  {
    INTERFACE_PLUGIN_VERSION,
    "LIRC interface v" VERSION,
    "Hubert Chan",
    NULL,
    interface_lirc_init,
    interface_lirc_start,
    interface_lirc_running,
    interface_lirc_stop,
    interface_lirc_close
  };

}

extern "C" {

interface_plugin *interface_plugin_info()
{
  return &default_plugin;
}

}

// end of ap-lirc.cc
